@@ -52,7 +52,9 @@ class DBusMockObject(dbus.service.Object):
         props: A property_name (string) → property (Variant) map with initial
                properties on "interface"
         logfile: When given, method calls will be logged into that file name;
-                 if None, logging will be written to stdout.
+                 if None, logging will be written to stdout. Note that you can
+                 also query the called methods over D-BUS with GetCalls() and
+                 GetMethodCalls().
         '''
         super(self.__class__, self).__init__(bus_name, path)
 
@@ -69,6 +71,7 @@ class DBusMockObject(dbus.service.Object):
             self.logfile = open(logfile, 'w')
         else:
             self.logfile = None
+        self.call_log = []
 
     def __del__(self):
         if self.logfile:
@@ -192,6 +195,12 @@ class DBusMockObject(dbus.service.Object):
               global "objects" variable, which is a dictionary mapping object
               paths to DBusMockObject instances.
 
+              For keeping state across method calls, you are free to use normal
+              Python members of the "self" object, which will be persistant for
+              the whole mock's life time. E. g. you can have a method with
+              "self.my_state = True", and another method that returns it with
+              "ret = self.my_state".
+
               When specifying '', the method will not do anything (except
               logging) and return None.
         '''
@@ -336,7 +345,7 @@ class DBusMockObject(dbus.service.Object):
         m.append(signature=signature, *args)
         args = m.get_args_list()
 
-        fn = lambda self, *args: self.log('emit %s.%s: %s' % (interface, name, args))
+        fn = lambda self, *args: self.log('emit %s.%s%s' % (interface, name, self.format_args(args)))
         fn.__name__ = str(name)
         dbus_fn = dbus.service.signal(interface)(fn)
         dbus_fn._dbus_signature = signature
@@ -344,6 +353,34 @@ class DBusMockObject(dbus.service.Object):
 
         dbus_fn(self, *args)
 
+    @dbus.service.method(MOCK_IFACE,
+                         in_signature='',
+                         out_signature='a(tsav)')
+    def GetCalls(self):
+        '''List all the logged calls since the last call to ClearCalls().
+
+        Return a list of (timestamp, method_name, args_list) tuples.
+        '''
+        return self.call_log
+
+    @dbus.service.method(MOCK_IFACE,
+                         in_signature='s',
+                         out_signature='a(tav)')
+    def GetMethodCalls(self, method):
+        '''List all the logged calls of a particular method.
+
+        Return a list of (timestamp, args_list) tuples.
+        '''
+        return [(row[0], row[2]) for row in self.call_log if row[1] == method]
+
+    @dbus.service.method(MOCK_IFACE,
+                         in_signature='',
+                         out_signature='')
+    def ClearCalls(self):
+        '''Empty the log of mock call signatures.'''
+
+        self.call_log = []
+
     def mock_method(self, interface, dbus_method, in_signature, *args, **kwargs):
         '''Master mock method.
 
@@ -362,6 +399,7 @@ class DBusMockObject(dbus.service.Object):
         args = m.get_args_list()
 
         self.log(dbus_method + self.format_args(args))
+        self.call_log.append((int(time.time()), str(dbus_method), args))
 
         code = self.methods[interface][dbus_method][2]
         if code:
@@ -374,6 +412,8 @@ class DBusMockObject(dbus.service.Object):
         '''Format a D-BUS argument tuple into an appropriate logging string.'''
 
         def format_arg(a):
+            if isinstance(a, dbus.Boolean):
+                return str(bool(a))
             if isinstance(a, dbus.Byte):
                 return str(int(a))
             if isinstance(a, int) or isinstance(a, long):
